
# q-expansion of newform 304.10.a.c, downloaded from the LMFDB on 26 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 304
weight = 10
poly_data = [85948, -4552, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[-1, 3, 0], 1], [[-3043, 24, 1], 3]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0], [-1, -1, 0], [162, 2, 1], [4439, 14, 10], [-18656, 386, 43], [52727, -204, 67], [-209697, 858, -564], [130321, 0, 0], [308209, -5110, -3289], [-3279673, -13893, -3628], [-454876, 29685, -8839], [-763030, 3757, 14761], [-4299860, 13925, -15333], [7459546, -106246, 9441], [-19632122, -15016, -44215], [2923543, -299391, -106356], [-5475433, 71558, -55609], [-42281260, 143100, -21479], [96025103, 461027, 162254], [-26040758, -583150, -105562], [-185980615, 492830, -464234], [106740674, -517415, -65849], [-143497154, 44200, -330414], [145896548, 2108807, 726611], [-128317382, 2792874, 1872892], [63206202, -1578298, -1518942], [-14382666, 1978098, 160922], [-701376663, -8546018, -111947], [-189450263, 6298789, -2035626], [599284838, -2527559, -81323], [1055878666, -9702419, 8215417], [-2357040130, -9142084, -1871763], [1255608491, 30254436, -990382], [-1506545594, -20379734, 236249], [-3584748498, 21210474, -2393605], [3903756678, -29215636, -3939324], [-3888443782, -59480408, 4917044], [12099671668, -18036498, -8767866], [-4061040648, 44520441, 2588431], [-1977733290, -54713586, 18608774], [-6432373412, -66851909, 1671925], [-13638580142, -10146858, -6101754], [-9714882821, 99776424, 33306236], [-5597475446, -47498267, -29224185], [1118523412, 108097380, 13369438], [26346755, 21718648, -18852396], [-785723773, 243986654, 35717591], [2240989218, 51294221, -12781219], [-10435948261, -27020978, 49974825], [6324344452, 17585234, 37438593], [-30205274048, -58135676, 19412397], [-3598984437, 343131132, 1555388], [18124973420, 417513721, 19877619], [-9247503720, -110448302, 87925217], [-6068739396, -196439650, 27247730], [-31294863738, 85899414, -149586261], [22951469766, -486635211, -101549833], [49800135785, -241657656, -3479467], [-74482243794, 613473368, 115106913], [17608324756, 105384488, -46732626], [23783460520, -285781100, 259385781], [-161183049111, 221859930, 34759587], [-127692137172, -482171883, -90454743], [-18363530961, -86784856, 130900406], [-53132048477, -530654614, -16612085], [37582829129, 733939772, -200887397], [24309444461, 857466103, 1211392], [-26439908452, -1002161672, 123013284], [87946226100, 52650266, 429362685], [-317934531432, 79970744, -143641185], [-204741153449, -1450326218, 188442135], [-66951417143, 189404278, -812790348], [342568887852, 784496992, 145467084], [-129636041411, 354940706, 285805991], [303316102975, -176875880, 510860837], [58012419342, 794953631, 684610685], [-36397809864, 3460532784, -56058439], [47333145418, -3840192832, -524395167], [479310628368, -315799427, 247470461], [348165152592, 2862425993, -702704695], [-358272986308, 534318626, -1307497294], [742945361659, 440897033, 196680988], [-207984187710, 467502858, 1491162964], [-155197637758, -380385717, -918321329], [104625942296, -1428502607, 937655855], [282365419416, -579049258, -1030354651], [152353608362, 1942528398, -828073592], [443521092173, -1661072460, -1059411274], [903427737578, -894750596, -39471835], [272517513106, -3251830084, -2479117475], [1473762135196, 3254274474, -96392355], [763085477620, -513075576, -98848], [-201900580118, -1210703950, 1354740730], [801669401340, -4354182050, 1142113126], [-1425983491974, -5561725844, 58283669], [1639630966833, -1081951008, -1052455927], [-1550497648230, -820499166, 1115225676], [-861999612716, 8538177027, 2768697675], [-1321917097115, 11067914100, 2974499159], [708574509440, -11838713808, -1983480639], [1813407140780, 7496518524, -2218077772], [-2274387652698, -11835458624, 1210431553], [-522137630824, 3794211989, -4532010229], [1631361237888, -8530096700, -1545380836], [-1359970199084, -7075255056, 2490648632], [-1459214231973, -14519035262, -2629496702], [776380094250, -21380102748, -1551127411], [-499663081334, 6799694102, 448464834], [-3793686313768, -9984634159, -354694215], [373415277672, -15592670152, -8447808546], [-1108355394506, -19885976472, 660442470], [4353164889780, -5533325734, 2063500281], [-544313156188, -7956468706, 2437342309], [1144594304570, -19390784562, -7685521636], [232479951546, -21447393430, -560822391], [-82192101622, 6237921278, 5547984296], [-1844244283784, 18467830136, 1177204913], [2053861559683, 12709173668, 8330928912], [-2546638693306, 5322196970, 9632298159], [2010516093669, 15349983862, 5176586085], [-3317580404663, -3482960940, 4657446985], [-829574549280, -80156876, 3593166150], [-417889330983, -5599368971, 8730496240], [1086238598932, -24312832218, 8030699886], [-1754643796968, 9115146852, -537658513], [-3660710610312, -11680527910, -5739778484], [-1290190999986, -29392654428, 252249656], [-860492039553, 13212394322, -6254185162], [494654490815, -21005590482, 5161220820], [-9018251688316, -12084515146, -5759255392], [-3689235929214, -6727168810, -11642929991], [-5200626413860, -19988291715, -10973063939], [-2991708452656, -30278893814, 88949762], [-13100038187912, -12477958350, 2946430179], [9887906081711, -37674895132, -8185628990], [-5654663424639, 56329234016, 19060953850], [-875669635483, -9038584132, 7060426019], [10898648208667, 18281019092, -1622665289], [18883872427, 17800634077, 24793695332], [-2863825082435, 42146143174, 18027741644], [-7961912540015, 35730345484, -6873013223], [1722643901998, -64881108296, -16932836251], [-10079442955867, 7849912846, -29861470280], [-10688996700973, 17587136769, -22267569800], [-15776483528207, -69517322702, 6012955927], [8571638075380, -3654527098, 23366783952], [7310168405258, -9398516458, 6779089630], [4366990512376, -24084723074, 1049656980], [4841382497148, -38727291932, 6474667517], [-13077452640994, 20743501367, -15484654721], [-11775301082623, 30426656464, 12712676611], [15771533523464, -11643098950, -22743107727], [10396588287556, 33844047818, 22611089139], [-3050358071922, 65211362624, 7378497908], [1373593025385, -24938868373, 1020550044], [-3972333121788, -115620184176, -29633325062], [-17820221460261, 41193465236, -20936058313], [24212072142927, 14655616118, 11143332715], [19256532864713, -46441220754, -17802581794], [7028170577859, -82161739586, 36552324607], [-2823899648720, 39207061456, -31612657180], [-19227884039894, 106700119719, 17516294701], [22988100415104, -40121743812, 43786093020], [-15568009115172, -108919976288, -12718545840], [2119435959940, 123817269261, -42836950011], [15877420828894, 191779342168, -17591949150], [10887973430464, 181117114511, -36167714251], [16151937549030, -41324194120, 68499011909], [13247590723740, 61922329319, 18214116275], [12999665138182, 211588249186, 24104548583], [21497861684529, -152909493507, -68860076288], [-11250587890585, 101720933289, 761150840], [25310756272912, 107787323572, 12629182996], [33798449010545, 104355178440, -33376836813], [19257457037488, 28938857200, 18524869700], [-3444211188484, 14668866146, 29369819031], [-6538160070592, -150844825070, 77387468462], [-28663899288680, 105045357298, -74581782447], [16434914761656, -184588919024, -86251946288], [18920159787234, -26723995658, 15637849681], [1625687320637, -155266657254, -29896875970], [-56761717075139, 129759531294, 32363583255], [-36030032573294, -31543025712, 32696369300], [17914276909820, 157964722890, -90608895184], [-7735421156045, -7946560526, 89464368650], [13968547526164, 148306926256, -13602956053], [20608324982381, 58407956892, -5127537113], [-33135962835324, 222763017436, 70833130974], [29206971631734, 198779262811, -57589572301], [-13168942334271, 90650613000, 85963659369], [5182442300614, -172150314597, -53449384105], [30076936361558, 299675839752, 44253975571], [6927895119776, -27377871477, 51473603847], [-30586123570251, -539456499351, 85952321224], [-43922338872964, -79100103164, -150281339120], [15727258607948, -41434892554, 124067019622], [-58316976776927, -384083736604, 60125696704], [-19273745374424, -259671870206, 78421873288], [26675996098869, -322448504974, -124554753739], [-45856884477523, 455412464876, 31280399832], [-86166739979678, 154563000724, 117866927194], [-21662767999604, -456939837493, 84356351077], [43999115234583, 238647443807, 25333078022], [21873627787912, 314912196875, 93605951241], [-36805169917458, -229879869062, 134492491084], [-70083857884338, 16138641050, -152226283293], [-66807904655405, 573145803084, 69791870154], [-42168285034413, 522323115810, -71266299967], [-70002316124491, -409000962928, 80397423044], [63041353603163, 217775861422, -106286319937], [37384906150347, -69215697272, 4334717501], [-39882452462484, 663654550346, 59239994564], [-5704362444098, 326990596216, -80412958961], [-28993222529639, -209302117010, -14218653665], [2727969967800, 362736294519, 109237564069], [19501112380130, 188266669331, -10390020445], [17062965112573, 464273059216, 72501160369], [-42504980322494, -67596089156, 287675376050], [-116997935864418, -440346428443, 50110208331], [-37551043416480, 183229895382, 197766439335], [-76927462593002, 279734680450, -263428713446], [55962337718990, -716920691321, -201758113581], [19532686798778, -61949043161, -4496913709], [-28102795402577, 840427616736, 147034684794], [-97418090923291, 265403540244, 69180857927], [7944644977924, 232785417850, 306410893791], [-66745889513102, 742546221330, -42480370302], [69183413759386, -657892550585, -47656421051], [-39856250920786, 1029060567700, 136086971548], [142551939630472, -815638009502, 21780274811], [-96523433158332, -215325385458, -40798625441], [-51779312546649, 344825032967, -304816367302], [163514628686272, 709780332253, 5533769733], [-71566084618278, 633013801966, 338649233582], [-30906989114180, 117980812350, 138081790629], [33819686332269, 212822408406, 301957621876], [-37224305014504, 1079446242182, 340918869849], [7252778688082, 473736691680, 262366763071], [-29733556743522, 210758338684, 64767094218], [-149849184388756, 439648830419, -300151357377], [-207063283858664, 619740258858, -181764028008], [139548443690292, 539087514664, -237281967841], [-14801451674117, -1122531198034, -19405441755], [117004028352531, -114780873428, -284819473011], [-50775481457600, 641873081701, -178032662745], [-17714572810302, -1231721236890, 406605295875], [-20290798544890, 1238711614422, 182464014271], [71244315805316, -723087040233, 357044388175], [111976943467540, -1690520917584, -35677637148], [77480464871075, -560911435384, -136792265894], [100565535625190, 811821475456, 101423304596], [22195417004187, 509921239514, 361297583666], [138845450811733, 883325366564, -158553233887], [245559126728522, -727353461793, -371143599563], [171574813695838, -262416422502, -187242520039], [-66043857358890, -1921881081946, 109344628688], [-161396985219148, 529786590650, 473725799353], [-49388602583824, -376103147435, -554096914993], [315615305311902, 627775220913, -85480458075], [-103333639554306, 234011685566, -863359819494], [72808467114152, 2375620194555, 52223071125], [14254636255760, 1560332654306, -78932521864], [129654880101458, 299167338824, 319541251375], [32618554419223, -526728333644, 727523823317], [-359527304194576, -261568819564, -144816105983], [189344672496971, -23172542595, -324185797240], [-10072211208631, 940661368949, 34244242130], [-154792922132517, -3006414968250, 396955126258], [23116253056245, 1063581599192, -95580488745], [-185111074463176, -691531750144, -322632972851], [368870249114247, -904246068531, -563312468124], [56778486393113, -813264864471, 36259667492], [-129388733435623, -509070194038, -970075000734], [94214570228553, 2193534268184, -637457261130], [428681173717850, 42304210545, 75744790047], [-77287881793137, 1075353196926, 890316718505], [-181559164604364, 1357371669216, 52119604469], [120309904577711, 1524844197247, -577691480646], [69871249636616, 471440551090, 649506083804], [-56411657353174, -2196516322288, -178128823457], [242467567336782, -488023986276, 938111710088], [-149684086901579, -122782398628, 206632598697], [206811636414775, 1760525007682, 650464815220], [416320816810295, -612574876510, -262695008865], [-645498444598834, -385150950396, -452749889681], [-83208259429982, -1116210150776, 1065108434391], [-185007765103979, -926126826884, -913268964014], [-379677495916554, -2187817730322, 265368370360], [179133803453745, -1661917118258, -693696611060], [-139815249476798, -1065901586268, 256905334710], [-49374337140734, -2950795647866, -118763372038], [324448507907406, -1789907643588, -993117004069], [-980299186717130, -529104779672, 133324403278], [-63239310445664, -518825391028, 893380453714], [-242007906126718, 3970026981042, -129372270982], [-505914513004680, -579259871392, 153246649710], [-121380030346362, -2106444895708, 720322652986], [99741141931676, -347163365756, 1368898405449], [35228376696561, -1055597495407, 1104914775930], [-331314580164836, -820697979896, 552464111251], [118053067736016, 561147035276, 1172934606415], [253097234662345, 4509224338056, 258216931415], [-45350492662705, -425506927974, -269849865202], [-132542354692553, 4944311032709, 234006827700], [-293542663872270, 719788291416, 1115430918353], [-190561496524006, 3212485261707, 1343084160669], [44021016818011, 800404192984, -425647312285], [-149720970764383, 1011247190793, -118127261804], [-160032304978702, 989572116980, -1738994476445], [-678229434576928, -3320275297692, -3660699294], [-190470963935974, -1575369501182, 880461084353], [312576351567366, -3748110790536, -1255045578628], [31008690939678, -1519443973478, 549612535928], [541615009762976, -1989163751701, 997665528433], [24015448933608, -229013525634, 2225284976390], [-8129059996792, -1859199381655, 307934387149], [-421926681541060, 2427597022184, 545998505123], [-1200225238518302, -1977728455923, -59548601005], [-77187987186493, 2163691468866, 2272099508022], [-496463500653912, -24870588122, 1094535663805], [-190664299952707, 6312767324895, 1419741290422], [542956898321619, -1241087518872, 1934042204634], [-293026565164249, 3688102809134, -468640937009], [113481564645718, 2775876412203, 540011279741], [474478563588225, -2406245663112, -719285325577], [79363539595942, 650865565951, -2056242451915], [464688650048451, -640573768682, -1618178665583], [-212691331142551, 4968978487791, -1140396427602], [-561245500127900, 3770124041409, -702675226905], [662012401593898, -5002660771860, -164881124459], [-978983455532518, 4475891206288, 1017872389323], [-29765256321279, 1125485403564, -1174261925553], [654482262336389, -5471754948488, -1735867251919], [310712933494676, 3560589224530, -809392399481], [-925696217810624, 1142005111348, -817940631235], [-129184651887490, 2055098230908, 2129367519433], [-619687059633555, -101508075843, -1890315414610], [719420461903908, -2151739642971, 41231498243], [-71645356137241, -4280425439622, -266139345554], [858168020569639, -2847229437532, -27219535401], [-750756104851091, 2647443456567, 106283377732], [-44623287220517, 8378759361106, 1048224535352], [-1668370110008993, 3771959315953, 1337200819546], [-46087431341574, 827810191211, -3078182860079], [585165284091043, 2532618833448, -635042544775], [-1629063976578356, 4360198169778, 638219001198], [-289499096833310, 397032960850, -4305482811110], [1262904537615621, 2769900406095, -1063201434030], [-318145988664382, -5421760203552, 1621114103858], [-363631899509170, -1935005143858, -171763292221], [20676451235292, -4479006356519, -226892864449], [-1299695937833802, -5205066107948, 1473696032742], [674198032065898, 7360569000282, -1195370160103], [-52761573017788, -12353457658336, -785453450826], [746397544462310, -6393978532755, -823208985825], [552796481675248, -10205779530634, -2538743103066], [751476101057069, 6213752105238, 201678238399], [-946921049357448, -5860245191784, 1251516473148], [-658503963273301, 764620456824, 1593610224130], [1128481818892402, -4486003537541, -2964103810837], [-655165972155914, 2492224917966, 2068277512915], [-497223987277442, -8537669275416, -789138860308], [-1583503760963206, 9581541429221, 3085452463765], [197597427164107, 1148587871699, 4766386891574], [1306797350831084, 5916093404692, -2252178246221], [-309037228504848, -5235897475757, -4707075384899], [-2875365390211746, 478139388696, -765656238604], [-487540273742742, -12671262572628, -1269363682812], [743116697495490, 182293973363, 3810172026481], [1301101649288486, -2128200741880, 2208395483378], [800324177323341, 9209118392805, -35429991544], [-19626405112357, 2251990113534, -1664113632252], [-85904196174673, 7136747458509, -2334529200414], [745863004544030, 9591586804120, 2340157473723], [121227165208942, 12239625578534, -1150883909133], [-237025372998623, -18816281144404, -1060298191219], [-335176744417128, 131151318646, 580022351062], [1314919587898324, 61817735642, -995307176337], [911423065363057, 5296930452040, -928497317938], [36334944620468, 7080623262794, -297201148258], [-537483169619342, -14087755214984, -2955190538224], [145033806497352, 4251128124374, -1470551323889], [1487934064827073, 6825555123722, 3005430579034], [-1001511779027844, 2087427757362, -3728515057689], [-1573484808137837, -1662331055357, -4355079185780], [-2012043250172110, 3686475327216, 329763329722], [-1731987128864629, -8280092367552, 1904443808791], [934321163375176, -6765204869088, -2988577336538], [78846695879144, 11631750947550, 2914579545393], [-1048339812440332, -4935061186019, -7580820404969], [3954871196801542, 5512055298194, 107608300414], [-149767698942319, 1477702297138, 4336586411645], [-2213916934275538, 6946103588108, 2872409087938], [1053910398111832, -7617299286556, 2024914841583], [-686148767845664, 5947972377610, -4613804851120], [-298417164438418, 20984100106254, -720651924310], [-930081429191506, -104236590363, 1166431571731], [-1910523254130436, 4880121283369, -4502189563629], [-1534245581041248, 10621872901783, 5970486637217], [-1959556646336068, 5453580668498, 892028673484], [-1200128397717750, -2291810567669, -3905990288499], [1669746121775696, -7002325692150, -247847195577], [786806901607878, 13537014023532, -1997065637278], [-212566670970924, -12788097296444, 1068320327198], [-1357271107036310, -695020751087, -3099306505377], [1024974090647535, 13413222180212, -2116256590774], [-1365142036210368, -3878121373288, 465011733259], [-469285756897600, -16295073573371, -5499657754457], [-2641251782973071, -4752792367945, 1477809973266], [3429235213789190, 7571614304640, -1991165434707], [-663064597985096, -2160498898809, 1670559889659], [-2443568066437028, 10517844052360, 2754912684937], [-2444459568739909, 10108408303060, -220794258119], [316570850818600, -16988165857836, -5144766311749], [3010069304320139, -1359473886176, 1509590904837], [-2444226073719192, -3396957949266, 1670594599008], [-43273908085926, -25864530868722, 3581896349704], [-3377450294049700, 11996710513237, -2494073621863], [-1144321929032627, -1802394513584, 2436850248623], [3909220548614468, -3086091578144, -3378765778414], [997271605198006, -5442889596593, 2215920974297], [2537647931856093, 9194528185333, -4217868971624], [-2349444810349447, 5677927245552, -5340194083492], [-1252582511006571, -14424035544409, -5021697163662], [864056656373206, -11571769118076, 4556917874086], [218359899946533, 8003676833670, -3803293946921], [4686542198131835, -5398229317449, 6931057229068], [4610172797941203, 6796768192600, 813573904576], [-372566792067678, -17235801778744, -823149925485], [-1505770912553217, 7380258579022, 7913926649140]]
